package org.jeecg.modules.tbOrder.controller;

import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.tbOrder.entity.TbOrder;
import org.jeecg.modules.tbOrder.service.ITbOrderService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: tb_order
 * @Author: jeecg-boot
 * @Date:   2024-12-12
 * @Version: V1.0
 */
@Api(tags="tb_order")
@RestController
@RequestMapping("/tbOrder/tbOrder")
@Slf4j
public class TbOrderController extends JeecgController<TbOrder, ITbOrderService> {
	@Autowired
	private ITbOrderService tbOrderService;
	
	/**
	 * 分页列表查询
	 *
	 * @param tbOrder
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "tb_order-分页列表查询")
	@ApiOperation(value="tb_order-分页列表查询", notes="tb_order-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<TbOrder>> queryPageList(TbOrder tbOrder,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<TbOrder> queryWrapper = QueryGenerator.initQueryWrapper(tbOrder, req.getParameterMap());
		Page<TbOrder> page = new Page<TbOrder>(pageNo, pageSize);
		IPage<TbOrder> pageList = tbOrderService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param tbOrder
	 * @return
	 */
	@AutoLog(value = "tb_order-添加")
	@ApiOperation(value="tb_order-添加", notes="tb_order-添加")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody TbOrder tbOrder) {
		tbOrderService.save(tbOrder);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param tbOrder
	 * @return
	 */
	@AutoLog(value = "tb_order-编辑")
	@ApiOperation(value="tb_order-编辑", notes="tb_order-编辑")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody TbOrder tbOrder) {
		tbOrderService.updateById(tbOrder);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "tb_order-通过id删除")
	@ApiOperation(value="tb_order-通过id删除", notes="tb_order-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		tbOrderService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "tb_order-批量删除")
	@ApiOperation(value="tb_order-批量删除", notes="tb_order-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.tbOrderService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "tb_order-通过id查询")
	@ApiOperation(value="tb_order-通过id查询", notes="tb_order-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<TbOrder> queryById(@RequestParam(name="id",required=true) String id) {
		TbOrder tbOrder = tbOrderService.getById(id);
		if(tbOrder==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(tbOrder);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param tbOrder
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, TbOrder tbOrder) {
        return super.exportXls(request, tbOrder, TbOrder.class, "tb_order");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, TbOrder.class);
    }

}
