package org.jeecg.modules.tbPremium.controller;

import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.tbPremium.entity.TbPremium;
import org.jeecg.modules.tbPremium.service.ITbPremiumService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: 保险费账单表
 * @Author: jeecg-boot
 * @Date:   2024-12-12
 * @Version: V1.0
 */
@Api(tags="保险费账单表")
@RestController
@RequestMapping("/tbPremium/tbPremium")
@Slf4j
public class TbPremiumController extends JeecgController<TbPremium, ITbPremiumService> {
	@Autowired
	private ITbPremiumService tbPremiumService;
	
	/**
	 * 分页列表查询
	 *
	 * @param tbPremium
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "保险费账单表-分页列表查询")
	@ApiOperation(value="保险费账单表-分页列表查询", notes="保险费账单表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<TbPremium>> queryPageList(TbPremium tbPremium,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<TbPremium> queryWrapper = QueryGenerator.initQueryWrapper(tbPremium, req.getParameterMap());
		Page<TbPremium> page = new Page<TbPremium>(pageNo, pageSize);
		IPage<TbPremium> pageList = tbPremiumService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param tbPremium
	 * @return
	 */
	@AutoLog(value = "保险费账单表-添加")
	@ApiOperation(value="保险费账单表-添加", notes="保险费账单表-添加")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody TbPremium tbPremium) {
		tbPremiumService.save(tbPremium);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param tbPremium
	 * @return
	 */
	@AutoLog(value = "保险费账单表-编辑")
	@ApiOperation(value="保险费账单表-编辑", notes="保险费账单表-编辑")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody TbPremium tbPremium) {
		tbPremiumService.updateById(tbPremium);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "保险费账单表-通过id删除")
	@ApiOperation(value="保险费账单表-通过id删除", notes="保险费账单表-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		tbPremiumService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "保险费账单表-批量删除")
	@ApiOperation(value="保险费账单表-批量删除", notes="保险费账单表-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.tbPremiumService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "保险费账单表-通过id查询")
	@ApiOperation(value="保险费账单表-通过id查询", notes="保险费账单表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<TbPremium> queryById(@RequestParam(name="id",required=true) String id) {
		TbPremium tbPremium = tbPremiumService.getById(id);
		if(tbPremium==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(tbPremium);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param tbPremium
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, TbPremium tbPremium) {
        return super.exportXls(request, tbPremium, TbPremium.class, "保险费账单表");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, TbPremium.class);
    }

}
