package org.jeecg.modules.tbTask.controller;

import java.util.Arrays;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.tbTask.entity.TbTask;
import org.jeecg.modules.tbTask.service.ITbTaskService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.jeecg.common.aspect.annotation.AutoLog;

 /**
 * @Description: tb_task
 * @Author: jeecg-boot
 * @Date:   2024-12-12
 * @Version: V1.0
 */
@Api(tags="tb_task")
@RestController
@RequestMapping("/tbTask/tbTask")
@Slf4j
public class TbTaskController extends JeecgController<TbTask, ITbTaskService> {
	@Autowired
	private ITbTaskService tbTaskService;
	
	/**
	 * 分页列表查询
	 *
	 * @param tbTask
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "tb_task-分页列表查询")
	@ApiOperation(value="tb_task-分页列表查询", notes="tb_task-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<TbTask>> queryPageList(TbTask tbTask,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<TbTask> queryWrapper = QueryGenerator.initQueryWrapper(tbTask, req.getParameterMap());
		Page<TbTask> page = new Page<TbTask>(pageNo, pageSize);
		IPage<TbTask> pageList = tbTaskService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param tbTask
	 * @return
	 */
	@AutoLog(value = "tb_task-添加")
	@ApiOperation(value="tb_task-添加", notes="tb_task-添加")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody TbTask tbTask) {
		tbTaskService.save(tbTask);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param tbTask
	 * @return
	 */
	@AutoLog(value = "tb_task-编辑")
	@ApiOperation(value="tb_task-编辑", notes="tb_task-编辑")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody TbTask tbTask) {
		tbTaskService.updateById(tbTask);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "tb_task-通过id删除")
	@ApiOperation(value="tb_task-通过id删除", notes="tb_task-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		tbTaskService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "tb_task-批量删除")
	@ApiOperation(value="tb_task-批量删除", notes="tb_task-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.tbTaskService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "tb_task-通过id查询")
	@ApiOperation(value="tb_task-通过id查询", notes="tb_task-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<TbTask> queryById(@RequestParam(name="id",required=true) String id) {
		TbTask tbTask = tbTaskService.getById(id);
		if(tbTask==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(tbTask);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param tbTask
    */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, TbTask tbTask) {
        return super.exportXls(request, tbTask, TbTask.class, "tb_task");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, TbTask.class);
    }

}
